﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.Bandits {
  public abstract class BanditPolicy<TPolicyActionInfo> : IPolicy<TPolicyActionInfo> where TPolicyActionInfo : IPolicyActionInfo {
    public IEnumerable<int> Actions { get; private set; }
    private readonly int numInitialActions;

    protected BanditPolicy(int numActions) {
      this.numInitialActions = numActions;
      Actions = Enumerable.Range(0, numActions).ToArray();
    }

    public abstract int SelectAction();
    public abstract void UpdateReward(int action, double reward);

    public virtual void DisableAction(int action) {
      Debug.Assert(Actions.Contains(action));

      Actions = Actions.Where(a => a != action).ToArray();
    }

    public virtual void Reset() {
      Actions = Enumerable.Range(0, numInitialActions).ToArray();
    }

    public abstract void PrintStats();
  }
}
