﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.Bandits {
  public class UCB1TunedPolicy : IPolicy {

    public int SelectAction(Random random, IEnumerable<IPolicyActionInfo> actionInfos) {
      var myActionInfos = actionInfos.OfType<MeanAndVariancePolicyActionInfo>().ToArray(); // TODO: performance
      int bestAction = -1;
      double bestQ = double.NegativeInfinity;
      int totalTries = myActionInfos.Where(a => !a.Disabled).Sum(a => a.Tries);

      for (int a = 0; a < myActionInfos.Length; a++) {
        if (myActionInfos[a].Disabled) continue;
        if (myActionInfos[a].Tries == 0) return a;

        var sumReward = myActionInfos[a].SumReward;
        var tries = myActionInfos[a].Tries;

        var avgReward = sumReward / tries;
        var q = avgReward + Math.Sqrt((Math.Log(totalTries) / tries) * Math.Min(1.0 / 4, V(myActionInfos[a], totalTries))); // 1/4 is upper bound of bernoulli distributed variable
        if (q > bestQ) {
          bestQ = q;
          bestAction = a;
        }
      }
      Debug.Assert(bestAction > -1);
      return bestAction;
    }

    public IPolicyActionInfo CreateActionInfo() {
      return new MeanAndVariancePolicyActionInfo();
    }

    private double V(MeanAndVariancePolicyActionInfo actionInfo, int totalTries) {
      var s = actionInfo.Tries;
      return actionInfo.RewardVariance + Math.Sqrt(2 * Math.Log(totalTries) / s);
    }

    public override string ToString() {
      return "UCB1TunedPolicy";
    }
  }
}
