﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.Bandits {
  /* Kocsis et al. Bandit based Monte-Carlo Planning */
  public class UCTPolicy : IPolicy {
    private readonly double c;

    public UCTPolicy(double c = 1.0) {
      this.c = c;
    }


    public int SelectAction(Random random, IEnumerable<IPolicyActionInfo> actionInfos) {
      var myActionInfos = actionInfos.OfType<DefaultPolicyActionInfo>().ToArray(); // TODO: performance
      int bestAction = -1;
      double bestQ = double.NegativeInfinity;
      int totalTries = myActionInfos.Where(a => !a.Disabled).Sum(a => a.Tries);

      for (int a = 0; a < myActionInfos.Length; a++) {
        if (myActionInfos[a].Disabled) continue;
        if (myActionInfos[a].Tries == 0) return a;
        var q = myActionInfos[a].SumReward / myActionInfos[a].Tries + 2 * c * Math.Sqrt(Math.Log(totalTries) / myActionInfos[a].Tries);
        if (q > bestQ) {
          bestQ = q;
          bestAction = a;
        }
      }
      Debug.Assert(bestAction > -1);
      return bestAction;
    }

    public IPolicyActionInfo CreateActionInfo() {
      return new DefaultPolicyActionInfo();
    }

    public override string ToString() {
      return string.Format("UCTPolicy({0:F2})", c);
    }
  }
}
